#ifndef VEC3_HPP
#define VEC3_HPP

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

#include <iostream>
using std::ostream;

#include <iomanip>
using std::setw;

class vec3
{
public:
	// member data
	double x,y,z;

	// constructor
	vec3(double x_, double y_, double z_) : x(x_), y(y_), z(z_) {}
	
	vec3& operator+=(const vec3 &u)
	{
		x += u.x;
		y += u.y;
		z += u.z;
		return *this;
	}

	friend vec3 operator+(const vec3 &u, const vec3 &v)
	{
		return vec3(u.x+v.x,u.y+v.y,u.z+v.z);
	}

	friend vec3 operator-(const vec3 &u, const vec3 &v)
	{
		return vec3(u.x-v.x,u.y-v.y,u.z-v.z);
	}

	friend vec3 operator*(const vec3 &v, double scale)
	{
		return vec3(v.x*scale,v.y*scale,v.z*scale);
	}

	friend vec3 operator/(const vec3 &v, double scale)
	{
		return vec3(v.x/scale,v.y/scale,v.z/scale);
	}

	friend ostream& operator<<(ostream& os, const vec3 &u)
	{
		os.precision(8);
		os << setw(20) << u.x
		    << setw(20) << u.y
		    << setw(20) << u.z;
		return os;
	}
};

#endif // VEC3_HPP