#include <cstdlib>
using std::exit;

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif

void initialize_bundle(void);
void update_bundle(void);
void draw_bundle(void);

GLfloat angle1 = 0;   /* in degrees */
GLfloat angle2 = 0;   /* in degrees */
GLfloat angle3 = 0;   /* in degrees */
GLfloat cameraPos = 600.;
bool moving = false;
enum rotation_style_e { ROT_XZ, ROT_YZ, ROT_XY, ZOOM };
rotation_style_e rotation_style = ZOOM;
int startx, starty;

void changeSize(int w, int h) {

    // Prevent a divide by zero, when window is too short
    // (you can't make a window of zero width).
    if (h == 0)
        h = 1;

    float ratio = 1.0* w / h;

    // Reset the coordinate system before modifying
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();

    // Set the viewport to be the entire window
    glViewport(0, 0, w, h);

    // Set the correct perspective.
    gluPerspective(45,ratio,1,1000); // (field of view angle, aspect ratio, near clipping, far clipping)
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gluLookAt(0.0,0.0,cameraPos,       // camera position
              0.0,0.0,-1.0,      // focal point
              0.0f,1.0f,0.0f);   // up direction    
}

enum {
    MENU_ROT_XZ,
    MENU_ROT_YZ,
    MENU_ROT_XY,
    MENU_ROT_RESET,
    MENU_ZOOM,
    MENU_QUIT
};

void menuSelect(int value)
{
    switch (value) {
        case MENU_ROT_XZ:
            rotation_style = ROT_XZ;
            break;
        case MENU_ROT_YZ:
            rotation_style = ROT_YZ;
            break;
        case MENU_ROT_XY:
            rotation_style = ROT_XY;
            break;
        case MENU_ROT_RESET:
            angle1 = 0.0;
            angle2 = 0.0;
            angle3 = 0.0;
            break;
        case MENU_ZOOM:
            rotation_style = ZOOM;
            break;
        case MENU_QUIT:
            exit(0);
            break;
    }
    glutPostRedisplay();
}

void mouse(int button, int state, int x, int y)
{
	if (button == GLUT_LEFT_BUTTON) 
	{
		if (state == GLUT_DOWN) 
		{
			moving = true;
			startx = x;
			starty = y;
		}
		if (state == GLUT_UP) 
		{
			moving = false;
		}
	}
}

void motion(int x, int y)
{
	if (moving) 
	{
            switch (rotation_style)
              {
                case ROT_XZ:
                    angle3 += GLfloat(x - startx);
                    angle1 += GLfloat(y - starty);
                    break;
                case ROT_YZ:
                    angle3 += GLfloat(y - starty);
                    angle2 += GLfloat(x - startx);
                    break;
                case ROT_XY:
                    angle2 += GLfloat(x - startx);
                    angle1 += GLfloat(y - starty);
                    break;
                case ZOOM:
                    cameraPos *= 1.0+GLfloat(y - starty)/300.0;
              }
            startx = x;
            starty = y;
            glutPostRedisplay();
	}
}

const int delay = 1; // milliseconds

void display()
{

    if (rotation_style == ZOOM)
      {
        //glMatrixMode(GL_MODELVIEW);
        glLoadIdentity();

        gluLookAt(0.0,0.0,cameraPos,       // camera position
                  0.0,0.0,-1.0,      // focal point
                  0.0f,1.0f,0.0f);   // up direction
      }        

    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	glPushMatrix();
	
	glRotatef(angle1, 1.0, 0.0, 0.0);
	glRotatef(angle2, 0.0, 1.0, 0.0);
	glRotatef(angle3, 0.0, 0.0, 1.0);


    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	draw_bundle();
	
	glPopMatrix();
	glutSwapBuffers();
}

void keyboard(unsigned char key, int x, int y)
{
	if (key == 'q')
		exit(0);
}

void animate(int)
{
	update_bundle();
	glutPostRedisplay();
	glutTimerFunc(delay,animate,0);
}

int main(int argc, char **argv)
{
	initialize_bundle();

	glutInit(&argc,argv);
	glutInitDisplayMode (GLUT_RGB | GLUT_DOUBLE | GLUT_DEPTH);
	glutInitWindowSize (640, 640);
	glutInitWindowPosition (100, 100);
	glutCreateWindow("Chaotic Dynamics");
	glutDisplayFunc(display);
	glutKeyboardFunc(keyboard);	
	glutMouseFunc(mouse);
	glutMotionFunc(motion);
	glutTimerFunc(delay,animate,0);
	glutCreateMenu(menuSelect);
	glutAddMenuEntry("Rotate x-z", MENU_ROT_XZ);
	glutAddMenuEntry("Rotate y-z", MENU_ROT_YZ);
	glutAddMenuEntry("Rotate x-y", MENU_ROT_XY);
	glutAddMenuEntry("Rotate reset", MENU_ROT_RESET);
	glutAddMenuEntry("Zoom", MENU_ZOOM);
	glutAddMenuEntry("Quit", MENU_QUIT);
	glutAttachMenu(GLUT_RIGHT_BUTTON);

	glutReshapeFunc(changeSize);
	glutMainLoop();
}
